@@ -34,8 +34,10 @@
 #include <time.h>
 #include <fcntl.h>
 #include <sys/select.h>
+#include <poll.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #ifdef USE_GSSAPI
@@ -118,6 +120,14 @@ static void user2_handler( int sig )
         suspend = 0;
 }
 
+/*
+ * SIGCHLD handler: reap exiting processes
+ */
+static void child_handler(int sig)
+{
+	while (waitpid(-1, NULL, WNOHANG) > 0)
+		; /* empty */
+}
 /*
  * Handlers for various events coming back from the remote server.
  * Return -1 if the remote dispatcher should exit.
@@ -290,7 +300,7 @@ int main(int argc, char *argv[])
 {
 	event_t *e;
 	struct sigaction sa;
-	int rc;
+	int rc, q_len;
 
 	/* Register sighandlers */
 	sa.sa_flags = 0;
@@ -302,6 +312,8 @@ int main(int argc, char *argv[])
 	sigaction(SIGHUP, &sa, NULL);
 	sa.sa_handler = user2_handler;
 	sigaction(SIGUSR2, &sa, NULL);
+	sa.sa_handler = child_handler;
+	sigaction(SIGCHLD, &sa, NULL);
 	if (load_config(&config, CONFIG_FILE))
 		return 6;
 
@@ -395,11 +407,12 @@ int main(int argc, char *argv[])
 	} while (stop == 0);
 	close(sock);
 	free_config(&config);
+	q_len = queue_length();
 	destroy_queue();
 	if (stop)
 		syslog(LOG_NOTICE, "audisp-remote is exiting on stop request");
 
-	return 0;
+	return q_len ? 1 : 0;
 }
 
 #ifdef USE_GSSAPI
@@ -409,7 +422,7 @@ int main(int argc, char *argv[])
    are what we're interested in, but the network sees the tokens. The
    protocol we use for transferring tokens is to send the length first,
    four bytes MSB first, then the token data. We return nonzero on error. */
-static int recv_token (int s, gss_buffer_t tok)
+static int recv_token(int s, gss_buffer_t tok)
 {
 	int ret;
 	unsigned char lenbuf[4];
@@ -430,11 +443,15 @@ static int recv_token (int s, gss_buffer_t tok)
 		| ((uint32_t)(lenbuf[1] & 0xFF) << 16)
 		| ((uint32_t)(lenbuf[2] & 0xFF) << 8)
 		|  (uint32_t)(lenbuf[3] & 0xFF));
-
+	if (len > MAX_AUDIT_MESSAGE_LENGTH) {
+		syslog(LOG_ERR,
+			"GSS-API error: event length exceeds MAX_AUDIT_LENGTH");
+		return -1;
+	}
 	tok->length = len;
 	tok->value = (char *) malloc(tok->length ? tok->length : 1);
 	if (tok->length && tok->value == NULL) {
-		syslog(LOG_ERR, "Out of memory allocating token data %d %x",
+		syslog(LOG_ERR, "Out of memory allocating token data %zd %zx",
 				tok->length, tok->length);
 		return -1;
 	}
@@ -901,9 +918,17 @@ static int ar_write (int sk, const void *buf, int len)
 
 static int ar_read (int sk, void *buf, int len)
 {
-	int rc = 0, r;
+	int rc = 0, r, timeout = config.max_time_per_record * 1000;
+	struct pollfd pfd;
+
+	pfd.fd=sk;
+	pfd.events=POLLIN | POLLPRI | POLLHUP | POLLERR | POLLNVAL;
 	while (len > 0) {
 		do {
+			// reads can hang if cable is disconnected
+			int prc = poll(&pfd, (nfds_t) 1, timeout);
+			if (prc <= 0)
+				return -1;
 			r = read(sk, buf, len);
 		} while (r < 0 && errno == EINTR);
 		if (r < 0) {
