@@ -434,6 +434,18 @@ int audit_rule_fieldpair(struct audit_rule *rule, const char *pair, int flags)
 		case AUDIT_DIR:
 		case AUDIT_FILTERKEY:
 			return -10;
+                case AUDIT_ARG0...AUDIT_ARG3:
+			vlen = strlen(v);
+			if (isdigit((char)*(v)))
+				rule->values[rule->field_count] =
+					strtoul(v, NULL, 0);
+			else if (vlen >= 2 && *(v)=='-' &&
+						(isdigit((char)*(v+1))))
+				rule->values[rule->field_count] =
+					strtol(v, NULL, 0);
+			else
+				return -21;
+			break;
                 case AUDIT_DEVMAJOR...AUDIT_INODE:
                 case AUDIT_SUCCESS:
 			if (flags != AUDIT_FILTER_EXIT)
@@ -454,7 +466,12 @@ int audit_rule_fieldpair(struct audit_rule *rule, const char *pair, int flags)
 			if (!isdigit((char)*(v)))
 				return -21;
 
-			rule->values[rule->field_count] = strtol(v, NULL, 0);
+			if (field == AUDIT_INODE) 
+				rule->values[rule->field_count] =
+						strtoul(v, NULL, 0);
+			else
+				rule->values[rule->field_count] =
+						strtol(v, NULL, 0);
 			break;
 	}
 	++rule->field_count;
