@@ -34,7 +34,9 @@
 #include "gen_tables.h"
 #include "private.h"
 
+#ifdef WITH_ALPHA
 #include "alpha_tables.h"
+#endif
 #include "i386_tables.h"
 #include "ia64_tables.h"
 #include "ppc_tables.h"
@@ -63,7 +65,9 @@ static const struct int_transtab elftab[] = {
     { MACH_PPC,     AUDIT_ARCH_PPC    },
     { MACH_S390X,   AUDIT_ARCH_S390X  },
     { MACH_S390,    AUDIT_ARCH_S390   },
+#ifdef WITH_ALPHA
     { MACH_ALPHA,   AUDIT_ARCH_ALPHA  }
+#endif
 };
 #define AUDIT_ELF_NAMES (sizeof(elftab)/sizeof(elftab[0]))
 
@@ -107,9 +111,11 @@ int audit_name_to_syscall(const char *sc, int machine)
 		case MACH_S390:
 			found = s390_syscall_s2i(sc, &res);
 			break;
+#ifdef WITH_ALPHA
 	        case MACH_ALPHA:
 			found = alpha_syscall_s2i(sc, &res);
 			break;
+#endif
 		default:
 			return -1;
 	}
@@ -136,8 +142,10 @@ const char *audit_syscall_to_name(int sc, int machine)
 			return s390x_syscall_i2s(sc);
 		case MACH_S390:
 			return s390_syscall_i2s(sc);
+#ifdef WITH_ALPHA
 	        case MACH_ALPHA:
 			return alpha_syscall_i2s(sc);
+#endif
 	}
 	return NULL;
 }
