@@ -1,5 +1,5 @@
 /* x86_64_table.h --
- * Copyright 2005-08 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-09 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -315,4 +315,8 @@ _S(291, "epoll_create1")
 _S(292, "dup3")
 _S(293, "pipe2")
 _S(294, "inotify_init1")
+_S(295, "preadv")
+_S(296, "pwritev")
+_S(297, "rt_tgsigqueueinfo")
+_S(298, "perf_counter_open")
 
