@@ -40,6 +40,9 @@ const char *aulookup_uid(uid_t uid, char *buf, size_t size);
 void aulookup_destroy_uid_list(void);
 const char *aulookup_gid(gid_t gid, char *buf, size_t size);
 void aulookup_destroy_gid_list(void);
+char *unescape(char *buf);
+int is_hex_string(const char *str);
+void print_tty_data(const char *val);
 
 #endif
 
