@@ -1,6 +1,6 @@
 # Common utilities.
 #
-# Copyright (C) 2007, 2008 Red Hat, Inc.  All rights reserved.
+# Copyright (C) 2007, 2008, 2009 Red Hat, Inc.  All rights reserved.
 # This copyrighted material is made available to anyone wishing to use, modify,
 # copy, or redistribute it subject to the terms and conditions of the GNU
 # General Public License v.2.  This program is distributed in the hope that it
@@ -25,11 +25,12 @@ import lists
 __all__ = ('ParsingError',
            'audit_machine_id',
            'connect_and_run',
+           'filetype_string',
            'is_ids_key', 'is_row_separator',
            'key_string',
            'modal_error_dialog', 'msgtype_string',
-           'parse_elf', 'parse_ids_key', 'parse_msgtype', 'parse_syscall',
-           'parse_unsigned',
+           'parse_elf', 'parse_filetype', 'parse_ids_key', 'parse_msgtype',
+           'parse_syscall', 'parse_unsigned',
            'set_combo_entry_text', 'set_sensitive_all', 'syscall_string',
            'tree_model_delete', 'tree_model_move_down', 'tree_model_move_up',
            'write_new_file')
@@ -144,8 +145,9 @@ def parse_elf(string):
     '''
     try:
         arch = int(string)
-        m = audit.audit_elf_to_machine(arch)
-        if m < 0:
+        try:
+            m = audit.audit_elf_to_machine(arch)
+        except OSError:
             raise ParsingError(_('Unknown architecture %d') % arch)
     except ValueError:
         if string.lower() == 'b64':
@@ -160,11 +162,23 @@ def parse_elf(string):
             if _machine_bits[m] != 32:
                 raise ParsingError(_('32-bit architecture not supported'))
         else:
-            m = audit.audit_name_to_machine(string)
-            if m == -1:
+            try:
+                m = audit.audit_name_to_machine(string)
+            except OSError:
                 raise ParsingError(_('Unknown architecture "%s"') % string)
     return m
 
+def parse_filetype(string):
+    '''Parse file type string.
+
+    Return file type ID.  Raise ParsingError on error.
+
+    '''
+    try:
+        return audit.audit_name_to_ftype(string)
+    except OSError:
+        raise ParsingError(_('Unknown file type "%s"') % string)
+
 def is_ids_key(s):
     '''Return True if s is in the namespace reserved for IDS keys.'''
     return s.startswith('ids-')
@@ -194,12 +208,13 @@ def parse_msgtype(string):
 
     '''
     try:
-        return int(string)
+        v = int(string)
     except ValueError:
-        v = audit.audit_name_to_msg_type(string)
-        if v == -1:
+        try:
+            v = audit.audit_name_to_msg_type(string)
+        except OSError:
             raise ParsingError(_('Unknown message type "%s"') % string)
-        return v
+    return v
 
 def parse_syscall(string, machine_id):
     '''Parse a syscall name for the specified machine.
@@ -207,8 +222,9 @@ def parse_syscall(string, machine_id):
     Return a syscall number.  Raise ParsingError on error.
 
     '''
-    sc = audit.audit_name_to_syscall(string, machine_id)
-    if sc == -1:
+    try:
+        sc = audit.audit_name_to_syscall(string, machine_id)
+    except OSError:
         try:
             sc = int(string, 10)
         except ValueError:
@@ -235,6 +251,12 @@ def keys_string(keys):
     '''Return a string representing keys.'''
     return _(', ').join(keys)
 
+def filetype_string(filetype):
+    '''Return a string representing filetype.'''
+    s = audit.audit_ftype_to_name(filetype)
+    assert s is not None
+    return s
+
 def msgtype_string(msgtype):
     '''Return a string representing msgtype.'''
     s = audit.audit_msg_type_to_name(msgtype)
